@@ -853,14 +853,17 @@ struct OptDffPass : public Pass {
 		log("    opt_dff [-nodffe] [-nosdff] [-keepdc] [-sat] [selection]\n");
 		log("\n");
 		log("This pass converts flip-flops to a more suitable type by merging clock enables\n");
-		log("and synchronous reset multiplexers, removing unused control inputs, or potentially\n");
-		log("removes the flip-flop altogether, converting it to a constant driver.\n");
+		log("and synchronous reset multiplexers, removing unused control inputs, or\n");
+		log("potentially removes the flip-flop altogether, converting it to a constant\n");
+		log("driver.\n");
 		log("\n");
 		log("    -nodffe\n");
-		log("        disables dff -> dffe conversion, and other transforms recognizing clock enable\n");
+		log("        disables dff -> dffe conversion, and other transforms recognizing clock\n");
+		log("        enable\n");
 		log("\n");
 		log("    -nosdff\n");
-		log("        disables dff -> sdff conversion, and other transforms recognizing sync resets\n");
+		log("        disables dff -> sdff conversion, and other transforms recognizing sync\n");
+		log("        resets\n");
 		log("\n");
 		log("    -simple-dffe\n");
 		log("        only enables clock enable recognition transform for obvious cases\n");
